@@ -103,7 +103,7 @@ bool FontEngine::setFont (const Font &font) {
 		return true;
 
 	if (const char *path=font.path()) {
-		auto pf = dynamic_cast<const PhysicalFont*>(&font);
+		auto pf = font_cast<const PhysicalFont*>(&font);
 		if (setFont(path, font.fontIndex(), pf ? pf->getCharMapID() : CharMapID())) {
 			_currentFont = &font;
 			return true;
@@ -308,7 +308,7 @@ int FontEngine::getCharMapIDs (vector<CharMapID> &charmapIDs) const {
 	if (_currentFace) {
 		for (int i=0; i < _currentFace->num_charmaps; i++) {
 			FT_CharMap charmap = _currentFace->charmaps[i];
-			charmapIDs.emplace_back(CharMapID(charmap->platform_id, charmap->encoding_id));
+			charmapIDs.emplace_back(charmap->platform_id, charmap->encoding_id);
 		}
 	}
 	return charmapIDs.size();
