@@ -2,7 +2,7 @@
 ** GraphicsPath.hpp                                                     **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -20,8 +20,10 @@
 
 #pragma once
 
+#include <algorithm>
 #include <array>
 #include <cctype>
+#include <cmath>
 #include <deque>
 #include <ostream>
 #include <type_traits>
@@ -539,7 +541,7 @@ class GraphicsPath {
 			}
 		}
 
-		void arcto (double rx, double ry, double angle, bool laf, bool sweep, const Point &p) {
+		void arcto (T rx, T ry, double angle, bool laf, bool sweep, const Point &p) {
 			_commands.emplace_back(ArcTo{rx, ry, angle, laf, sweep, p});
 			_finalPoint = p;
 		}
@@ -596,7 +598,8 @@ class GraphicsPath {
 
 		/** Computes the bounding box of the current path.
 		 *  @param[out] bbox the computed bounding box */
-		void computeBBox (BoundingBox &bbox) const {
+		BoundingBox computeBBox () const {
+			BoundingBox bbox;
 			struct BBoxActions : IterationActions {
 				explicit BBoxActions (BoundingBox &bb) : bbox(bb) {}
 				void moveto (const Point &p) override {bbox.embed(p);}
@@ -609,6 +612,7 @@ class GraphicsPath {
 				BoundingBox &bbox;
 			} actions(bbox);
 			iterate(actions, false);
+			return bbox;
 		}
 
 		/** Checks whether the current path describes a dot/point only (with no extent).
@@ -659,24 +663,14 @@ class GraphicsPath {
 		bool operator == (const GraphicsPath &path) const {
 			if (size() != path.size())
 				return false;
-			auto it = _commands.begin();
-			for (const auto &cmd : path._commands) {
-				if (*it++ != cmd)
-					return false;
-			}
-			return true;
+			return std::equal(_commands.begin(), _commands.end(), path._commands.begin());
 		}
 
 		/** Returns true if this path differs from another one (command-wise). */
 		bool operator != (const GraphicsPath &path) const {
 			if (size() != path.size())
 				return true;
-			auto it = _commands.begin();
-			for (const auto &cmd : path._commands) {
-				if (*it++ != cmd)
-					return true;
-			}
-			return false;
+			return !std::equal(_commands.begin(), _commands.end(), path._commands.begin());
 		}
 
 		/** Iterates over all commands defining this path and calls the corresponding template methods.
@@ -687,7 +681,7 @@ class GraphicsPath {
 		 *  @param[in] actions template methods called by each iteration step
 		 *  @param[in] optimize if true, shorthand drawing commands (hlineto, vlineto,...) are considered */
 		void iterate (IterationActions &actions, bool optimize) const {
-			double eps = XMLString::DECIMAL_PLACES > 0 ? pow(10, -XMLString::DECIMAL_PLACES) : 1e-7;
+			double eps = XMLString::DECIMAL_PLACES > 0 ? std::pow(10, -XMLString::DECIMAL_PLACES) : 1e-7;
 			IterationVisitor visitor(actions, optimize, eps);
 			for (const CommandVariant &cmd : _commands) {
 				if (actions.quit())
@@ -729,7 +723,7 @@ class GraphicsPath {
 
 	private:
 		std::deque<CommandVariant> _commands; ///< sequence of path commands
-		WindingRule _windingRule;
+		WindingRule _windingRule = WindingRule::NON_ZERO;
 		Point _startPoint; ///< start point of final sub-path
 		Point _finalPoint; ///< final point reached by last command in path
 };
