@@ -2,7 +2,7 @@
 ** Message.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -74,7 +74,7 @@ void MessageStream::putChar (char c, ostream &os) {
 					_col++;
 			}
 			_nl = false;
-			if (!_nl || c != '\n')
+			if (c != '\n')
 				os << c;
 	}
 }
