@@ -2,7 +2,7 @@
 ** dvisvgm.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -64,7 +64,7 @@ using namespace std;
 ////////////////////////////////////////////////////////////////////////////////
 
 static string remove_path (string fname) {
-	fname = FileSystem::adaptPathSeperators(fname);
+	fname = FileSystem::ensureForwardSlashes(fname);
 	size_t slashpos = fname.rfind('/');
 	if (slashpos == string::npos)
 		return fname;
@@ -244,7 +244,7 @@ class VersionInfo {
 
 	protected:
 		void append (const string &name, const string &version) {
-			_versionPairs.emplace_back(pair<string,string>(name, version));
+			_versionPairs.emplace_back(name, version);
 		}
 
 	private:
@@ -349,7 +349,8 @@ static void set_variables (const CommandLine &cmdline) {
 		string msg = "unknown font format '"+cmdline.fontFormatOpt.value()+"' (supported formats: ";
 		for (const string &format : FontWriter::supportedFormats())
 			msg += format + ", ";
-		msg.erase(msg.end()-2);
+		msg.erase(msg.end()-2, msg.end());
+		msg += ")";
 		throw CL::CommandLineException(msg);
 	}
 	SVGTree::CREATE_USE_ELEMENTS = cmdline.noFontsOpt.value() < 1;
@@ -363,10 +364,18 @@ static void set_variables (const CommandLine &cmdline) {
 	PhysicalFont::KEEP_TEMP_FILES = cmdline.keepOpt.given();
 	PhysicalFont::METAFONT_MAG = max(1.0, cmdline.magOpt.value());
 	XMLString::DECIMAL_PLACES = max(0, min(6, cmdline.precisionOpt.value()));
+	XMLNode::KEEP_ENCODED_FILES = cmdline.keepOpt.given();
 	PsSpecialHandler::COMPUTE_CLIPPATHS_INTERSECTIONS = cmdline.clipjoinOpt.given();
 	PsSpecialHandler::SHADING_SEGMENT_OVERLAP = cmdline.gradOverlapOpt.given();
 	PsSpecialHandler::SHADING_SEGMENT_SIZE = max(1, cmdline.gradSegmentsOpt.value());
 	PsSpecialHandler::SHADING_SIMPLIFY_DELTA = cmdline.gradSimplifyOpt.value();
+	PsSpecialHandler::BITMAP_FORMAT = util::tolower(cmdline.bitmapFormatOpt.value());
+	if (!PSInterpreter::imageDeviceKnown(PsSpecialHandler::BITMAP_FORMAT)) {
+		ostringstream oss;
+		oss << "unknown image format '" << PsSpecialHandler::BITMAP_FORMAT << "'\nknown formats:\n";
+		PSInterpreter::listImageDeviceInfos(oss);
+		throw CL::CommandLineException(oss.str());
+	}
 	if (cmdline.optimizeOpt.given()) {
 		SVGOptimizer::MODULE_SEQUENCE = cmdline.optimizeOpt.value();
 		vector<string> modnames;
@@ -472,16 +481,20 @@ int main (int argc, char *argv[]) {
 	}
 	catch (DVIException &e) {
 		Message::estream() << "\nDVI error: " << e.what() << '\n';
+		return -1;
 	}
 	catch (PSException &e) {
 		Message::estream() << "\nPostScript error: " << e.what() << '\n';
+		return -2;
 	}
 	catch (SignalException &e) {
 		Message::wstream().clearline();
 		Message::wstream(true) << "execution interrupted by user\n";
+		return -3;
 	}
 	catch (exception &e) {
 		Message::estream(true) << e.what() << '\n';
+		return -4;
 	}
 	return 0;
 }
